import { Server } from 'socket.io'
import fs from 'fs'
import { getPublicPath } from '#application/storage'
import sharp from 'sharp'
import bcrypt from 'bcryptjs'
import { Tile } from '#entities/tile'
import { MapObject } from '#entities/mapObject'
import { SpriteAction } from '#entities/spriteAction'
import { Sprite } from '#entities/sprite'
import { Zone } from '#entities/zone'
import { ZoneEffect } from '#entities/zoneEffect'
import { User } from '#entities/user'
import { Character } from '#entities/character'
import CharacterTypeRepository from '#repositories/characterTypeRepository'
import CharacterHairRepository from '#repositories/characterHairRepository'
import ZoneRepository from '#repositories/zoneRepository'

export default class InitCommand {
  constructor(private readonly io: Server) {}

  public async execute(): Promise<void> {
    // Assets
    await this.importTiles()
    await this.importObjects()
    await this.createCharacterSprite()
    await this.createCharacterHair()
    // await this.createCharacterEquipment()

    // Zone
    await this.createZone()

    // User
    await this.createUser()

    // Stop process
    process.exit(0)
  }

  private async importTiles(): Promise<void> {
    for (const tile of fs.readdirSync(getPublicPath('tiles'))) {
      const newTile = new Tile()
      newTile.id = tile.split('.')[0] as any
      newTile.name = 'New tile'
      await newTile.save()
    }
  }

  private async importObjects(): Promise<void> {
    for (const object of fs.readdirSync(getPublicPath('objects'))) {
      const newMapObject = new MapObject()
      newMapObject.id = object.split('.')[0] as any
      newMapObject.name = 'New object'

      newMapObject.frameWidth = await sharp(getPublicPath('objects', object))
        .metadata()
        .then((metadata) => metadata.height) ?? 0

      newMapObject.frameHeight = await sharp(getPublicPath('objects', object))
        .metadata()
        .then((metadata) => metadata.width) ?? 0

      await newMapObject.save()
    }
  }

  private async createCharacterSprite(): Promise<void> {
    const characterSprite = new Sprite()
    characterSprite.id = '023d1e9d-f57f-4faa-8412-86c07107cf85'
    characterSprite.name = 'Character'

    const idleRightDownAction = new SpriteAction()
    idleRightDownAction.action = 'idle_right_down'
    idleRightDownAction.sprites = ['data:image/png;base64,...'] // Base64 sprite data
    idleRightDownAction.originX = 0
    idleRightDownAction.originY = 0
    idleRightDownAction.isAnimated = false
    idleRightDownAction.isLooping = false
    idleRightDownAction.frameWidth = 64
    idleRightDownAction.frameHeight = 94
    idleRightDownAction.frameRate = 0
    idleRightDownAction.sprite = characterSprite
    await idleRightDownAction.save()

    const idleLeftUpAction = new SpriteAction()
    idleLeftUpAction.action = 'idle_left_up'
    idleLeftUpAction.sprites = ['data:image/png;base64,...'] // Base64 sprite data
    idleLeftUpAction.originX = 0
    idleLeftUpAction.originY = 0
    idleLeftUpAction.isAnimated = false
    idleLeftUpAction.isLooping = false
    idleLeftUpAction.frameWidth = 64
    idleLeftUpAction.frameHeight = 94
    idleLeftUpAction.frameRate = 0
    idleLeftUpAction.sprite = characterSprite
    await idleLeftUpAction.save()

    const walkRightDownAction = new SpriteAction()
    walkRightDownAction.action = 'walk_right_down'
    walkRightDownAction.sprites = ['data:image/png;base64,...'] // Base64 sprite data array
    walkRightDownAction.originX = 0
    walkRightDownAction.originY = 0
    walkRightDownAction.isAnimated = true
    walkRightDownAction.isLooping = false
    walkRightDownAction.frameWidth = 64
    walkRightDownAction.frameHeight = 94
    walkRightDownAction.frameRate = 7
    walkRightDownAction.sprite = characterSprite
    await walkRightDownAction.save()

    const walkLeftUpAction = new SpriteAction()
    walkLeftUpAction.action = 'walk_left_up'
    walkLeftUpAction.sprites = ['data:image/png;base64,...'] // Base64 sprite data array
    walkLeftUpAction.originX = 0
    walkLeftUpAction.originY = 0
    walkLeftUpAction.isAnimated = true
    walkLeftUpAction.isLooping = false
    walkLeftUpAction.frameWidth = 64
    walkLeftUpAction.frameHeight = 94
    walkLeftUpAction.frameRate = 7
    walkLeftUpAction.sprite = characterSprite
    await walkLeftUpAction.save()

    await characterSprite.save()
  }

  private async createCharacterHair(): Promise<void> {
    const hairSprite = new Sprite()
    hairSprite.id = '922ee95f-1500-49c0-8ead-f8cc46dad136'
    hairSprite.name = 'Hair 1'

    const frontAction = new SpriteAction()
    frontAction.action = 'front'
    frontAction.sprites = ['data:image/png;base64,...'] // Base64 sprite data
    frontAction.originX = 0.5
    frontAction.originY = 5.34
    frontAction.isAnimated = false
    frontAction.isLooping = false
    frontAction.frameWidth = 64
    frontAction.frameHeight = 18
    frontAction.frameRate = 0
    frontAction.sprite = hairSprite
    await frontAction.save()

    const backAction = new SpriteAction()
    backAction.action = 'back'
    backAction.sprites = ['data:image/png;base64,...'] // Base64 sprite data
    backAction.originX = 0.5
    backAction.originY = 4.34
    backAction.isAnimated = false
    backAction.isLooping = false
    backAction.frameWidth = 64
    backAction.frameHeight = 22
    backAction.frameRate = 0
    backAction.sprite = hairSprite
    await backAction.save()

    await hairSprite.save()
  }

  private async createCharacterEquipment(): Promise<void> {
    const equipmentSprite = new Sprite()
    equipmentSprite.id = '5b3932dd-0791-4bb7-bb1e-da9833c3cc50'
    equipmentSprite.name = 'Male shirt'

    // Create actions similar to createCharacterSprite()
    // with appropriate sprite data and parameters
    const actions = [
      {
        action: 'idle_right_down',
        sprites: ['data:image/png;base64,...'],
        originX: 0,
        originY: 0,
        isAnimated: false,
        isLooping: false,
        frameWidth: 64,
        frameHeight: 94,
        frameRate: 0
      },
      // Add other actions...
    ]

    for (const actionData of actions) {
      const action = new SpriteAction()
      Object.assign(action, actionData)
      action.sprite = equipmentSprite
      await action.save()
    }

    await equipmentSprite.save()
  }

  private async createZone(): Promise<void> {
    const zone = new Zone()
    zone.name = 'New zone'
    zone.width = 100
    zone.height = 100
    zone.tiles = Array.from({ length: 100 }, () =>
      Array.from({ length: 100 }, () => 'a2fd8d6f-5042-437a-9c1e-c66b91ecc35b')
    )
    await zone.save()

    const effect = new ZoneEffect()
    effect.effect = 'light'
    effect.strength = 100
    effect.zone = zone
    await effect.save()
  }

  private async createUser(): Promise<void> {
    const user = new User()
    user.id = 1
    user.username = 'root'
    user.email = 'local@host'
    user.password = await bcrypt.hash('password', 10)
    user.online = false
    await user.save()

    const character = new Character()
    character.id = 1
    character.user = user
    character.name = 'root'
    character.role = 'gm'
    character.zone = await ZoneRepository.getFirst() ?? undefined
    character.characterType = await CharacterTypeRepository.getFirst() ?? undefined
    character.characterHair = await CharacterHairRepository.getFirst() ?? undefined
    await character.save()
  }
}